/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Action.h>

#include <sstream>

namespace pagliaio {

Action::Action() {
	//xmlInitParser();

	this->response = createEmptyResponse();
	this->responseEl = xmlDocGetRootElement(this->response);
}

Action::~Action() {
	//free the document
	xmlFreeDoc(this->response);

	/*
	 *Free the global variables that may
	 *have been allocated by the parser.
	 */
	//xmlCleanupParser();
}

xmlDocPtr Action::createEmptyResponse() {
	xmlDocPtr xmlResponse = xmlNewDoc(BAD_CAST "1.0");
	xmlNodePtr responseRoot = xmlNewNode(NULL, BAD_CAST "response");
    xmlDocSetRootElement(xmlResponse, responseRoot);

	return xmlResponse;
}

void Action::addErrorToResponse(xmlDocPtr responseDoc, std::string error) {
	if (responseDoc != 0) {
		xmlNodePtr responseEl = xmlDocGetRootElement(responseDoc);
		if (responseEl != 0) {
			xmlNewChild(responseEl, NULL, BAD_CAST "error", BAD_CAST error.c_str());
		}
	}
}

void Action::addErrorToResponse(std::string error) {
	return addErrorToResponse(this->response, error);
}

xmlNodePtr Action::addChildToResponse(xmlNodePtr child) {
	return xmlAddChild(this->responseEl, child);
}

void Action::addAttributeToResponse(std::string name, std::string value) {
	xmlNewProp(this->responseEl, BAD_CAST name.c_str(), BAD_CAST value.c_str());
}

std::string Action::toString() {
	xmlChar *xmlbuff;
	int buffersize;

	xmlDocDumpFormatMemory(this->response, &xmlbuff, &buffersize, 1);
	std::stringstream ssxml;
	ssxml << (char*) xmlbuff;

	xmlFree(xmlbuff);

	return ssxml.str();
}

char* Action::convertToMultiByte(const wchar_t* text) {
	size_t size = wcslen(text);

	char* cText = new char[size + 1];
	cText[size] = 0;

	wcstombs(cText, text, size);

	return cText;
}

}
