/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "include/Configuration.h"

namespace pagliaio {

Configuration::Configuration() {
	this->defaultField = "id";
	this->defaultOperator = "OR";
	this->defaultIndexType = Lucene::Field::INDEX_ANALYZED;
	this->defaultStoring = Lucene::Field::STORE_NO;
}

Configuration::~Configuration() {}

void Configuration::addIndex(std::string name, bool stored, std::string type) {
	Lucene::String nameUnicode = Lucene::StringUtils::toUnicode(name.c_str());
	return this->addIndex(nameUnicode, stored, type);
}

void Configuration::addIndex(Lucene::String name, bool stored, std::string type) {
	if (stored)
		this->indexes[name].stored = Lucene::Field::STORE_YES;
	else
		this->indexes[name].stored = Lucene::Field::STORE_NO;

	if (type == "analyzed")
		this->indexes[name].type = Lucene::Field::INDEX_ANALYZED;
	else if (type == "not_analyzed")
		this->indexes[name].type = Lucene::Field::INDEX_NOT_ANALYZED;
	else if (type == "no")
		this->indexes[name].type = Lucene::Field::INDEX_NO;
	else
		this->indexes[name].type = defaultIndexType;
}

Lucene::Field::Store Configuration::indexNeedStoring(std::string name) {
	Lucene::String nameUnicode = Lucene::StringUtils::toUnicode(name.c_str());
	return this->indexNeedStoring(nameUnicode);
}

Lucene::Field::Store Configuration::indexNeedStoring(Lucene::String name) {

	if (this->indexes.count(name))
		return this->indexes[name].stored;
	else
		return this->defaultStoring; //this is the default
}

Lucene::Field::Index Configuration::getIndexType(std::string name) {
	Lucene::String nameUnicode = Lucene::StringUtils::toUnicode(name.c_str());
	return this->getIndexType(nameUnicode);
}

Lucene::Field::Index Configuration::getIndexType(Lucene::String name) {
	if (this->indexes.count(name))
		return this->indexes[name].type;
	else
		return this->defaultIndexType;
}
void Configuration::setDefaultIndexType(std::string type) {
	if (type == "analyzed")
		this->defaultIndexType = Lucene::Field::INDEX_ANALYZED;
	else if (type == "not_analyzed")
		this->defaultIndexType = Lucene::Field::INDEX_NOT_ANALYZED;
	else if (type == "no")
		this->defaultIndexType = Lucene::Field::INDEX_NO;
	else
		this->defaultIndexType = Lucene::Field::INDEX_ANALYZED; //this is the default
}

void Configuration::setDefaultStoring(std::string storing) {
	if (storing == "yes") {
		this->defaultStoring = Lucene::Field::STORE_YES;
	}
	else
		this->defaultStoring = Lucene::Field::STORE_NO; //this is the default
}

} /* namespace pagliaio */
