/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Configuration.h>

#include "include/PagliaioQueryParser.h"
#include <string>

using namespace Lucene;
using namespace std;

extern pagliaio::Configuration configuration;

namespace pagliaio {

PagliaioQueryParser::PagliaioQueryParser(LuceneVersion::Version matchVersion,
		                                 const String& field,
		                                 AnalyzerPtr analyzer): QueryParser(matchVersion, field, analyzer) {}

PagliaioQueryParser::PagliaioQueryParser(QueryParserCharStreamPtr stream): QueryParser(stream) {}

PagliaioQueryParser::PagliaioQueryParser(QueryParserTokenManagerPtr tokenMgr): QueryParser(tokenMgr) {}

QueryPtr PagliaioQueryParser::getFieldQuery(const String& field, const String& queryText) {
	if (configuration.getIndexType(field) != Field::INDEX_ANALYZED)
		return newTermQuery(newLucene<Term>(field, queryText));
	else
		return QueryParser::getFieldQuery(field, queryText);
}

QueryPtr PagliaioQueryParser::getFieldQuery(const String& field, const String& queryText, int32_t slop) {
	if (configuration.getIndexType(field) != Field::INDEX_ANALYZED)
			this->setLowercaseExpandedTerms(false);

	return QueryParser::getFieldQuery(field, queryText, slop);
}

QueryPtr PagliaioQueryParser::getRangeQuery(const String& field, const String& part1, const String& part2, bool inclusive) {
	if (configuration.getIndexType(field) != Field::INDEX_ANALYZED)
			this->setLowercaseExpandedTerms(false);

	return QueryParser::getRangeQuery(field, part1, part2, inclusive);
}

QueryPtr PagliaioQueryParser::getBooleanQuery(Collection<BooleanClausePtr> clauses) {
	return QueryParser::getBooleanQuery(clauses);
}

QueryPtr PagliaioQueryParser::getBooleanQuery(Collection<BooleanClausePtr> clauses, bool disableCoord) {
	return QueryParser::getBooleanQuery(clauses, disableCoord);
}

QueryPtr PagliaioQueryParser::getWildcardQuery(const String& field, const String& termStr) {
	if (configuration.getIndexType(field) != Field::INDEX_ANALYZED)
			this->setLowercaseExpandedTerms(false);

	return QueryParser::getWildcardQuery(field, termStr);
}

QueryPtr PagliaioQueryParser::getPrefixQuery(const String& field, const String& termStr) {
	if (configuration.getIndexType(field) != Field::INDEX_ANALYZED)
			this->setLowercaseExpandedTerms(false);

	return QueryParser::getPrefixQuery(field, termStr);
}

QueryPtr PagliaioQueryParser::getFuzzyQuery(const String& field, const String& termStr, double minSimilarity) {
	if (configuration.getIndexType(field) != Field::INDEX_ANALYZED)
			this->setLowercaseExpandedTerms(false);

	return QueryParser::getFuzzyQuery(field, termStr, minSimilarity);
}

PagliaioQueryParser::~PagliaioQueryParser() {}

}
