/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <cstdlib>
#include <cstring>
#include <map>
#include <string>

extern "C" {
#include <sys/time.h>
#include <stdint.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <stdarg.h>
#include <microhttpd.h>
}

#include <ActionFactory.h>

using namespace std;

string getRequestedAction(const char *url) {
	string urlStr = string(url);
	size_t slashPos = urlStr.find_last_of('/');

	return urlStr.substr(slashPos+1);
}

int httpRequestsHandler(void *cls, struct MHD_Connection *connection,
						const char *url,
						const char *method, const char *version,
						const char *upload_data,
						size_t *upload_data_size, void **con_cls)
{
	int ret = MHD_NO;

	string requestedAction = getRequestedAction(url);

	auto_ptr<pagliaio::Action> selectedAction = pagliaio::ActionFactory::resolveAction(requestedAction);

	if (selectedAction.get() != 0) {
		string xmlResponse = selectedAction->generateResponse(cls, connection, url, method, version, upload_data, upload_data_size, con_cls);

		if (xmlResponse.size() > 0) {
			MHD_Response* httpResponse = MHD_create_response_from_data(xmlResponse.size(), (void*)xmlResponse.c_str(), MHD_NO, MHD_YES);

			ret = MHD_queue_response(connection, MHD_HTTP_OK, httpResponse);
			MHD_destroy_response (httpResponse);
		}
		else { //� il primo passaggio di un POST
			ret = MHD_YES;
		}
	}
	else {
		//TODO restituire un messaggio di help ad esempio
		string xmlResponse = "//TODO restituire un messaggio di help ad esempio";
		MHD_Response* httpResponse = MHD_create_response_from_data(xmlResponse.size(), (void*)xmlResponse.c_str(), MHD_NO, MHD_YES);

		ret = MHD_queue_response(connection, MHD_HTTP_OK, httpResponse);
		MHD_destroy_response (httpResponse);
	}

	return ret;
}

int iterate_post(void* cls, enum MHD_ValueKind kind, const char* key, const char* filename, const char* contentType,
						const char* transferEncoding, const char *data,	uint64_t off, size_t size) {

	map<string, string>* args = (map<string, string>*) cls;

	if (args->count(string(key)) > 0) {
		string concatData = (*args)[string(key)] + string(data);
		(*args)[string(key)] = string(concatData);
	}
	else {
		(*args)[string(key)] = string(data);
	}

	return MHD_YES;
}

int iterate_get(void *cls, enum MHD_ValueKind kind, const char *key, const char *value) {
	std::map<std::string, std::string> *args = (std::map<std::string, std::string>*)cls;
	(*args)[std::string(key)] = std::string(value);
	return MHD_YES;
}
