/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PAGLIAIOSTANDARDANALYZER_H_
#define PAGLIAIOSTANDARDANALYZER_H_

#include <lucene++/Analyzer.h>

namespace pagliaio {

class LPPAPI PagliaioStandardAnalyzer: public Lucene::Analyzer {
public:
	/// Builds an analyzer with the default stop words ({@link #STOP_WORDS_SET}).
	/// @param matchVersion Lucene version to match.
	PagliaioStandardAnalyzer(Lucene::LuceneVersion::Version matchVersion);

	/// Builds an analyzer with the given stop words.
	/// @param matchVersion Lucene version to match.
	/// @param stopWords stop words
	PagliaioStandardAnalyzer(Lucene::LuceneVersion::Version matchVersion, Lucene::HashSet<Lucene::String> stopWords);

	/// Builds an analyzer with the stop words from the given file.
	/// @see WordlistLoader#getWordSet(const String&, const String&)
	/// @param matchVersion Lucene version to match.
	/// @param stopwords File to read stop words from.
	PagliaioStandardAnalyzer(Lucene::LuceneVersion::Version matchVersion, const Lucene::String& stopwords);

	/// Builds an analyzer with the stop words from the given reader.
	/// @see WordlistLoader#getWordSet(ReaderPtr, const String&)
	/// @param matchVersion Lucene version to match.
	/// @param stopwords Reader to read stop words from.
	PagliaioStandardAnalyzer(Lucene::LuceneVersion::Version matchVersion, Lucene::ReaderPtr stopwords);

	virtual ~PagliaioStandardAnalyzer();

	/// Constructs a {@link StandardTokenizer} filtered by a {@link StandardFilter}, a {@link LowerCaseFilter}
	/// and a {@link StopFilter}.
	virtual Lucene::TokenStreamPtr tokenStream(const Lucene::String& fieldName, Lucene::ReaderPtr reader);

	virtual Lucene::TokenStreamPtr reusableTokenStream(const Lucene::String& fieldName, Lucene::ReaderPtr reader);

protected:
	Lucene::HashSet<Lucene::String> stopSet;

	/// Specifies whether deprecated acronyms should be replaced with HOST type.
	bool replaceInvalidAcronym;
	bool enableStopPositionIncrements;

	Lucene::LuceneVersion::Version matchVersion;

	/// Construct an analyzer with the given stop words.
    void ConstructAnalyser(Lucene::LuceneVersion::Version matchVersion, Lucene::HashSet<Lucene::String> stopWords);
};

}
#endif /* PAGLIAIOSTANDARDANALYZER_H_ */
